// calculate tiles


const RADIUS_OPTS = [ 
	"const",
	"right",
	"left",
	"up",
	"down",
	"right-up",
	"right-down",
	"left-up",
	"left-down",
	"in",
	"out",
	"noise",
];

const RADIUS_DESC = {
	"const": "of constant size",
	"right": "getting larger towards the left",
	"left": "getting larger towards the left",
	"up": "getting larger towards the top",
	"down": "getting larger towards the bottom",
	"right-up": "getting larger towards the upper right",
	"right-down": "getting larger towards the lower right",
	"left-up": "getting larger towards the upper left",
	"left-down": "getting larger towards the lower left",
	"in": "getting larger in the centre",
	"out": "getting larger at the edges",
	"noise": "of random sizes",
}

function distance(dx, dy) {
	return Math.sqrt(dx ** 2 + dy ** 2);
}

function int_range(v1, v2) {
	const vs = [v1, v2];
	vs.sort((a, b) => a - b);
	const low = Math.floor(vs[0]);
	const high = Math.ceil(vs[1]);
	return [...Array(high - low + 1).keys()].map((i) => i + low);
}

class DotMaker {
	constructor(width) {
		this.width = width;
		this.cx = 0.5 * width;
		this.cy = 0.5 * width;
	}

	dots(m, n) {
		if( m - n === 0 ) {
			return [];
		}
		const ps = [];
		const is = int_range(-this.width, this.width / m)
		is.map((i) => {
			const js = int_range(m * i + (m - n) * this.width, m * i)
			js.map((j) => {
				const x = (j - m * i) / (m - n);
				const y = m * (x + i);
				if( x > 0 && y > 0 && x < this.width && y < this.width ) {
					ps.push({i:i, j:j, x:x, y:y});
				}
			});
		});
		return ps;
	}

	radius(d, func, maxr) {
	  	switch (func) {
	    	case "const":
	      		return maxr;
	    	case "right":
	      		return maxr * d.x / this.width;
	    	case "left":
	      		return maxr * (this.width - d.x) / this.width;
	    	case "down":
	      		return maxr * d.y / this.width;
	    	case "up":
	      		return maxr * (this.width - d.y) / this.width;
	      	case "right-up":
	      		return 0.5 * maxr * (d.x + this.width - d.y) / this.width;
	      	case "left-up":
	      		return 0.5 * maxr * (this.width - d.x + this.width - d.y) / this.width;
	      	case "right-down":
	      		return 0.5 * maxr * (d.x + d.y) / this.width;
	      	case "left-down":
	      		return 0.5 * maxr * (this.width - d.x + d.y) / this.width;
	      	case "out":
	      		return 2 * maxr * distance((d.x - this.cx), (d.y - this.cy)) / this.width;
	      	case "in":
	      		return 2 * maxr * (0.5 * this.width - distance((d.x - this.cx), (d.y - this.cy))) / this.width;

	      	// case "hyper-out":
	      	// 	return 2 * maxr * Math.abs(d.x - this.cx) (d.y - this.cy)) / this.width;
	      	// case "hyoer-in":
	      	// 	return 2 * maxr * (0.5 * this.width - distance((d.x - this.cx), (d.y - this.cy))) / this.width;

	      	case "noise":
	      		return maxr * Math.random();
	    	default:
	      		return maxr;
	  }
	}
}

export { RADIUS_OPTS, RADIUS_DESC, DotMaker };


